@@ -7,21 +7,23 @@ import (
 	"encoding/csv"
 	"encoding/json"
 	"fmt"
+	"io"
+	"log"
 	"os"
+	"reflect"
 	"strings"
 	"time"
 	"unicode"
 	"unicode/utf8"
 
-	"github.com/karrick/gows"
-	"github.com/turbot/go-kit/helpers"
-	"github.com/turbot/steampipe/pkg/error_helpers"
-
 	"github.com/jedib0t/go-pretty/v6/table"
 	"github.com/jedib0t/go-pretty/v6/text"
+	"github.com/karrick/gows"
 	"github.com/spf13/viper"
+	"github.com/turbot/go-kit/helpers"
 	"github.com/turbot/steampipe/pkg/cmdconfig"
 	"github.com/turbot/steampipe/pkg/constants"
+	"github.com/turbot/steampipe/pkg/error_helpers"
 	"github.com/turbot/steampipe/pkg/query/queryresult"
 	"golang.org/x/text/language"
 	"golang.org/x/text/message"
@@ -30,24 +32,33 @@ import (
 // ShowOutput displays the output using the proper formatter as applicable
 func ShowOutput(ctx context.Context, result *queryresult.Result, opts ...DisplayOption) int {
 	rowErrors := 0
-	config := NewDisplayConfiguration()
+	config := newDisplayConfiguration()
 	for _, o := range opts {
 		o(config)
 	}
 
-	switch cmdconfig.Viper().GetString(constants.ArgOutput) {
+	var timingResult *queryresult.TimingResult
+
+	outputFormat := cmdconfig.Viper().GetString(constants.ArgOutput)
+	switch outputFormat {
 	case constants.OutputFormatJSON:
-		rowErrors = displayJSON(ctx, result)
+		rowErrors, timingResult = displayJSON(ctx, result)
 	case constants.OutputFormatCSV:
-		rowErrors = displayCSV(ctx, result)
+		rowErrors, timingResult = displayCSV(ctx, result)
 	case constants.OutputFormatLine:
-		rowErrors = displayLine(ctx, result)
+		rowErrors, timingResult = displayLine(ctx, result)
 	case constants.OutputFormatTable:
-		rowErrors = displayTable(ctx, result)
+		rowErrors, timingResult = displayTable(ctx, result)
 	}
 
-	if config.timing {
-		fmt.Println(buildTimingString(result))
+	// show timing
+	if config.timing != constants.ArgOff && timingResult != nil {
+		str := buildTimingString(timingResult)
+		if viper.GetBool(constants.ConfigKeyInteractive) {
+			fmt.Println(str)
+		} else {
+			fmt.Fprintln(os.Stderr, str)
+		}
 	}
 	// return the number of rows that returned errors
 	return rowErrors
@@ -57,6 +68,7 @@ type ShowWrappedTableOptions struct {
 	AutoMerge        bool
 	HideEmptyColumns bool
 	Truncate         bool
+	OutputMirror     io.Writer
 }
 
 func ShowWrappedTable(headers []string, rows [][]string, opts *ShowWrappedTableOptions) {
@@ -67,7 +79,11 @@ func ShowWrappedTable(headers []string, rows [][]string, opts *ShowWrappedTableO
 
 	t.SetStyle(table.StyleDefault)
 	t.Style().Format.Header = text.FormatDefault
-	t.SetOutputMirror(os.Stdout)
+	if opts.OutputMirror == nil {
+		t.SetOutputMirror(os.Stdout)
+	} else {
+		t.SetOutputMirror(opts.OutputMirror)
+	}
 
 	rowConfig := table.RowConfig{AutoMerge: opts.AutoMerge}
 	colConfigs, headerRow := getColumnSettings(headers, rows, opts)
@@ -153,7 +169,104 @@ func getColumnSettings(headers []string, rows [][]string, opts *ShowWrappedTable
 	return colConfigs, headerRow
 }
 
-func displayLine(ctx context.Context, result *queryresult.Result) int {
+// getTerminalColumnsRequiredForString returns the length of the longest line in the string
+func getTerminalColumnsRequiredForString(str string) int {
+	colsRequired := 0
+	scanner := bufio.NewScanner(bytes.NewBufferString(str))
+	for scanner.Scan() {
+		line := scanner.Text()
+		runeCount := utf8.RuneCountInString(line)
+		if runeCount > colsRequired {
+			colsRequired = runeCount
+		}
+	}
+	return colsRequired
+}
+
+type jsonOutput struct {
+	Rows     []map[string]interface{}  `json:"rows"`
+	Metadata *queryresult.TimingResult `json:"metadata"`
+}
+
+func newJSONOutput() *jsonOutput {
+	return &jsonOutput{
+		Rows: make([]map[string]interface{}, 0),
+	}
+
+}
+
+func displayJSON(ctx context.Context, result *queryresult.Result) (int, *queryresult.TimingResult) {
+	rowErrors := 0
+	jsonOutput := newJSONOutput()
+
+	// define function to add each row to the JSON output
+	rowFunc := func(row []interface{}, result *queryresult.Result) {
+		record := map[string]interface{}{}
+		for idx, col := range result.Cols {
+			value, _ := ParseJSONOutputColumnValue(row[idx], col)
+			record[col.Name] = value
+		}
+		jsonOutput.Rows = append(jsonOutput.Rows, record)
+	}
+
+	// call this function for each row
+	count, err := iterateResults(result, rowFunc)
+	if err != nil {
+		error_helpers.ShowError(ctx, err)
+		rowErrors++
+		return rowErrors, nil
+	}
+
+	// now we have iterated the rows, get the timing
+	jsonOutput.Metadata = getTiming(result, count)
+
+	// display the JSON
+	encoder := json.NewEncoder(os.Stdout)
+	encoder.SetIndent("", " ")
+	encoder.SetEscapeHTML(false)
+	if err := encoder.Encode(jsonOutput); err != nil {
+		fmt.Print("Error displaying result as JSON", err)
+		return 0, nil
+	}
+	return rowErrors, jsonOutput.Metadata
+}
+
+func displayCSV(ctx context.Context, result *queryresult.Result) (int, *queryresult.TimingResult) {
+	rowErrors := 0
+	csvWriter := csv.NewWriter(os.Stdout)
+	csvWriter.Comma = []rune(cmdconfig.Viper().GetString(constants.ArgSeparator))[0]
+
+	if cmdconfig.Viper().GetBool(constants.ArgHeader) {
+		_ = csvWriter.Write(ColumnNames(result.Cols))
+	}
+
+	// print the data as it comes
+	// define function display each csv row
+	rowFunc := func(row []interface{}, result *queryresult.Result) {
+		rowAsString, _ := ColumnValuesAsString(row, result.Cols, WithNullString(""))
+		_ = csvWriter.Write(rowAsString)
+	}
+
+	// call this function for each row
+	count, err := iterateResults(result, rowFunc)
+	if err != nil {
+		error_helpers.ShowError(ctx, err)
+		rowErrors++
+		return rowErrors, nil
+	}
+
+	csvWriter.Flush()
+	if csvWriter.Error() != nil {
+		error_helpers.ShowErrorWithMessage(ctx, csvWriter.Error(), "unable to print csv")
+	}
+
+	// now we have iterated the rows, get the timing
+	timingResult := getTiming(result, count)
+
+	return rowErrors, timingResult
+}
+
+func displayLine(ctx context.Context, result *queryresult.Result) (int, *queryresult.TimingResult) {
 
 	maxColNameLength, rowErrors := 0, 0
 	for _, col := range result.Cols {
@@ -208,90 +321,19 @@ func displayLine(ctx context.Context, result *queryresult.Result) int {
 	}
 
 	// call this function for each row
-	if err := iterateResults(result, rowFunc); err != nil {
+	count, err := iterateResults(result, rowFunc)
+	if err != nil {
 		error_helpers.ShowError(ctx, err)
 		rowErrors++
-		return rowErrors
+		return rowErrors, nil
 	}
-	return rowErrors
+
+	// now we have iterated the rows, get the timing
+	timingResult := getTiming(result, count)
+	return rowErrors, timingResult
 }
 
-// getTerminalColumnsRequiredForString returns the length of the longest line in the string
-func getTerminalColumnsRequiredForString(str string) int {
-	colsRequired := 0
-	scanner := bufio.NewScanner(bytes.NewBufferString(str))
-	for scanner.Scan() {
-		line := scanner.Text()
-		runeCount := utf8.RuneCountInString(line)
-		if runeCount > colsRequired {
-			colsRequired = runeCount
-		}
-	}
-	return colsRequired
-}
-
-func displayJSON(ctx context.Context, result *queryresult.Result) int {
-	rowErrors := 0
-	jsonOutput := make([]map[string]interface{}, 0)
-
-	// define function to add each row to the JSON output
-	rowFunc := func(row []interface{}, result *queryresult.Result) {
-		record := map[string]interface{}{}
-		for idx, col := range result.Cols {
-			value, _ := ParseJSONOutputColumnValue(row[idx], col)
-			record[col.Name] = value
-		}
-		jsonOutput = append(jsonOutput, record)
-	}
-
-	// call this function for each row
-	if err := iterateResults(result, rowFunc); err != nil {
-		error_helpers.ShowError(ctx, err)
-		rowErrors++
-		return rowErrors
-	}
-	// display the JSON
-	encoder := json.NewEncoder(os.Stdout)
-	encoder.SetIndent("", " ")
-	encoder.SetEscapeHTML(false)
-	if err := encoder.Encode(jsonOutput); err != nil {
-		fmt.Print("Error displaying result as JSON", err)
-		return 0
-	}
-	return rowErrors
-}
-
-func displayCSV(ctx context.Context, result *queryresult.Result) int {
-	rowErrors := 0
-	csvWriter := csv.NewWriter(os.Stdout)
-	csvWriter.Comma = []rune(cmdconfig.Viper().GetString(constants.ArgSeparator))[0]
-
-	if cmdconfig.Viper().GetBool(constants.ArgHeader) {
-		_ = csvWriter.Write(ColumnNames(result.Cols))
-	}
-
-	// print the data as it comes
-	// define function display each csv row
-	rowFunc := func(row []interface{}, result *queryresult.Result) {
-		rowAsString, _ := ColumnValuesAsString(row, result.Cols, WithNullString(""))
-		_ = csvWriter.Write(rowAsString)
-	}
-
-	// call this function for each row
-	if err := iterateResults(result, rowFunc); err != nil {
-		error_helpers.ShowError(ctx, err)
-		rowErrors++
-		return rowErrors
-	}
-
-	csvWriter.Flush()
-	if csvWriter.Error() != nil {
-		error_helpers.ShowErrorWithMessage(ctx, csvWriter.Error(), "unable to print csv")
-	}
-	return rowErrors
-}
-
-func displayTable(ctx context.Context, result *queryresult.Result) int {
+func displayTable(ctx context.Context, result *queryresult.Result) (int, *queryresult.TimingResult) {
 	rowErrors := 0
 	// the buffer to put the output data in
 	outbuf := bytes.NewBufferString("")
@@ -302,7 +344,7 @@ func displayTable(ctx context.Context, result *queryresult.Result) int {
 	t.SetStyle(table.StyleDefault)
 	t.Style().Format.Header = text.FormatDefault
 
-	colConfigs := []table.ColumnConfig{}
+	var colConfigs []table.ColumnConfig
 	headers := make(table.Row, len(result.Cols))
 
 	for idx, column := range result.Cols {
@@ -339,7 +381,7 @@ func displayTable(ctx context.Context, result *queryresult.Result) int {
 	}
 
 	// iterate each row, adding each to the table
-	err := iterateResults(result, rowFunc)
+	count, err := iterateResults(result, rowFunc)
 	if err != nil {
 		// display the error
 		fmt.Println()
@@ -352,64 +394,188 @@ func displayTable(ctx context.Context, result *queryresult.Result) int {
 
 	// page out the table
 	ShowPaged(ctx, outbuf.String())
-	return rowErrors
+
+	// now we have iterated the rows, get the timing
+	timingResult := getTiming(result, count)
+
+	return rowErrors, timingResult
 }
 
-func buildTimingString(result *queryresult.Result) string {
-	timingResult := <-result.TimingResult
-	if timingResult == nil {
-		return ""
+func getTiming(result *queryresult.Result, count int) *queryresult.TimingResult {
+	if viper.GetString(constants.ArgTiming) == constants.ArgOff {
+		return nil
 	}
+	// now we have iterated the rows, get the timing
+	timingResult := <-result.TimingResult
+	// set rows returned
+	timingResult.RowsReturned = int64(count)
+	return timingResult
+}
+
+func buildTimingString(timingResult *queryresult.TimingResult) string {
 	var sb strings.Builder
 	// large numbers should be formatted with commas
 	p := message.NewPrinter(language.English)
 
-	milliseconds := float64(timingResult.Duration.Microseconds()) / 1000
-	seconds := timingResult.Duration.Seconds()
-	if seconds < 0.5 {
-		sb.WriteString(p.Sprintf("\nTime: %dms.", int64(milliseconds)))
-	} else {
-		sb.WriteString(p.Sprintf("\nTime: %.1fs.", seconds))
+	sb.WriteString(fmt.Sprintf("\nTime: %s.", getDurationString(timingResult.DurationMs, p)))
+	sb.WriteString(p.Sprintf(" Rows returned: %d.", timingResult.RowsReturned))
+	totalRowsFetched := timingResult.UncachedRowsFetched + timingResult.CachedRowsFetched
+	if totalRowsFetched == 0 {
+		// maybe there was an error retrieving timing - just display the basics
+		return sb.String()
 	}
 
-	if timingMetadata := timingResult.Metadata; timingMetadata != nil {
-		totalRows := timingMetadata.RowsFetched + timingMetadata.CachedRowsFetched
-		sb.WriteString(" Rows fetched: ")
-		if totalRows == 0 {
-			sb.WriteString("0")
-		} else {
-			if totalRows > 0 {
-				sb.WriteString(p.Sprintf("%d", timingMetadata.RowsFetched+timingMetadata.CachedRowsFetched))
-			}
-			if timingMetadata.CachedRowsFetched > 0 {
-				if timingMetadata.RowsFetched == 0 {
-					sb.WriteString(" (cached)")
-				} else {
-					sb.WriteString(p.Sprintf(" (%d cached)", timingMetadata.CachedRowsFetched))
-				}
-			}
+	sb.WriteString(" Rows fetched: ")
+	if totalRowsFetched == 0 {
+		sb.WriteString("0")
+	} else {
+
+		// calculate the number of cached rows fetched
+
+		sb.WriteString(p.Sprintf("%d", totalRowsFetched))
+
+		// were all cached
+		if timingResult.UncachedRowsFetched == 0 {
+			sb.WriteString(" (cached)")
+		} else if timingResult.CachedRowsFetched > 0 {
+			sb.WriteString(p.Sprintf(" (%d cached)", timingResult.CachedRowsFetched))
+		}
+	}
+
+	sb.WriteString(p.Sprintf(". Hydrate calls: %d.", timingResult.HydrateCalls))
+	if timingResult.ScanCount > 1 {
+		sb.WriteString(p.Sprintf(" Scans: %d.", timingResult.ScanCount))
+	}
+	if timingResult.ConnectionCount > 1 {
+		sb.WriteString(p.Sprintf(" Connections: %d.", timingResult.ConnectionCount))
+	}
+
+	if viper.GetString(constants.ArgTiming) == constants.ArgVerbose && len(timingResult.Scans) > 0 {
+		if err := getVerboseTimingString(&sb, p, timingResult); err != nil {
+			log.Printf("[WARN] Error getting verbose timing: %v", err)
 		}
-		sb.WriteString(p.Sprintf(". Hydrate calls: %d.", timingMetadata.HydrateCalls))
 	}
 
 	return sb.String()
 }
 
+func getDurationString(durationMs int64, p *message.Printer) string {
+	if durationMs < 500 {
+		return p.Sprintf("%dms", durationMs)
+	} else {
+		seconds := float64(durationMs) / 1000
+		return p.Sprintf("%.1fs", seconds)
+	}
+}
+
+func getVerboseTimingString(sb *strings.Builder, p *message.Printer, timingResult *queryresult.TimingResult) error {
+	scans := timingResult.Scans
+
+	// keep track of empty scans and do not include them separately in scan list
+	emptyScanCount := 0
+	scanCount := 0
+	// is this all scans or just the slowest
+	if len(scans) == int(timingResult.ScanCount) {
+		sb.WriteString("\n\nScans:\n")
+	} else {
+		sb.WriteString(fmt.Sprintf("\n\nSlowest %d scans:\n", len(scans)))
+	}
+
+	for _, scan := range scans {
+		if scan.RowsFetched == 0 {
+			emptyScanCount++
+			continue
+		}
+		scanCount++
+
+		cacheString := ""
+		if scan.CacheHit {
+			cacheString = " (cached)"
+		}
+		qualsString := formatQuals(scan)
+		limitString := ""
+		if scan.Limit != nil {
+			limitString = p.Sprintf(" Limit: %d.", *scan.Limit)
+		}
+
+		timeString := getDurationString(scan.DurationMs, p)
+		rowsFetchedString := p.Sprintf("%d", scan.RowsFetched)
+
+		sb.WriteString(p.Sprintf("  %d) %s.%s: Time: %s. Fetched: %s%s. Hydrates: %d.%s%s\n", scanCount, scan.Table, scan.Connection, timeString, rowsFetchedString, cacheString, scan.HydrateCalls, qualsString, limitString))
+	}
+	if emptyScanCount > 0 {
+
+		sb.WriteString(fmt.Sprintf("  %d…%d) Zero rows fetched.\n", scanCount+1, scanCount+emptyScanCount))
+	}
+	return nil
+}
+
+func formatQuals(scan *queryresult.ScanMetadataRow) string {
+	if len(scan.Quals) == 0 {
+		return ""
+	}
+
+	var b strings.Builder
+	for _, qual := range scan.Quals {
+		operator := qual.Operator
+		valueStr := formatQualValue(qual.Value)
+
+		if operator == "=" {
+
+			// Use reflection to check if qual.Value is an array or a slice
+			val := reflect.ValueOf(qual.Value)
+
+			if val.Kind() == reflect.Array || val.Kind() == reflect.Slice {
+				// Change operator to IN if it was "=" and the value is an array or slice
+				if operator == "=" {
+					operator = " IN "
+				}
+
+				// Build the string of array elements
+				valueElements := make([]string, val.Len())
+				for i := 0; i < val.Len(); i++ {
+					valueElements[i] = fmt.Sprintf("%s", formatQualValue(val.Index(i).Interface()))
+				}
+				valueStr = fmt.Sprintf("(%s)", strings.Join(valueElements, ", "))
+			} else {
+				// Use the original value if it's not an array or slice
+				valueStr = fmt.Sprintf("%v", qual.Value)
+			}
+		}
+
+		b.WriteString(fmt.Sprintf("%s%s%s, ", qual.Column, operator, valueStr))
+	}
+
+	// Remove the trailing comma and space
+	trimmedResult := strings.TrimRight(b.String(), ", ")
+
+	return fmt.Sprintf(" Quals: %s.", trimmedResult)
+}
+
+func formatQualValue(val any) string {
+	if str, ok := val.(string); ok {
+		return fmt.Sprintf("'%s'", str)
+	}
+	return fmt.Sprintf("%v", val)
+}
+
 type displayResultsFunc func(row []interface{}, result *queryresult.Result)
 
 // call func displayResult for each row of results
-func iterateResults(result *queryresult.Result, displayResult displayResultsFunc) error {
+func iterateResults(result *queryresult.Result, displayResult displayResultsFunc) (int, error) {
+	count := 0
 	for row := range *result.RowChan {
 		if row == nil {
-			return nil
+			return count, nil
 		}
 		if row.Error != nil {
-			return row.Error
+			return count, row.Error
 		}
 		displayResult(row.Data, result)
+		count++
 	}
 	// we will not get here
-	return nil
+	return count, nil
 }
 
 // DisplayErrorTiming shows the time taken for the query to fail
