@@ -95,7 +95,7 @@ func (c *SteampipeConfig) ConfigMap() map[string]interface{} {
 	return res
 }
 
-func (c *SteampipeConfig) SetOptions(opts options.Options) (errorsAndWarnings *error_helpers.ErrorAndWarnings) {
+func (c *SteampipeConfig) SetOptions(opts options.Options) (errorsAndWarnings error_helpers.ErrorAndWarnings) {
 	errorsAndWarnings = error_helpers.NewErrorsAndWarning(nil)
 
 	switch o := opts.(type) {
@@ -251,15 +251,15 @@ PluginOptions:
 func (c *SteampipeConfig) ConnectionsForPlugin(pluginLongName string, pluginVersion *version.Version) []*modconfig.Connection {
 	var res []*modconfig.Connection
 	for _, con := range c.Connections {
-		// extract stream from plugin
+		// extract constraint from plugin
 		ref := ociinstaller.NewSteampipeImageRef(con.Plugin)
-		org, plugin, stream := ref.GetOrgNameAndStream()
+		org, plugin, constraint := ref.GetOrgNameAndConstraint()
 		longName := fmt.Sprintf("%s/%s", org, plugin)
 		if longName == pluginLongName {
-			if stream == "latest" {
+			if constraint == "latest" {
 				res = append(res, con)
 			} else {
-				connectionPluginVersion, err := version.NewVersion(stream)
+				connectionPluginVersion, err := version.NewVersion(constraint)
 				if err != nil && connectionPluginVersion.LessThanOrEqual(pluginVersion) {
 					res = append(res, con)
 				}
